-- -------------------------------------------------------------------- --
-- Gui for La Vendetta Boss Mods written by LV|Nitram                   --
-- based on the default WoW UI Frames.                                  --
--                                                                      --
-- -------------------------------------------------------------------- --
--
-- ChangeLog (releases):
--
--  v0.75: 
-- 	Added Tooltip Support for the Dialog Functions
--
--  v0.80:
--  	Implementation of all Syncronization Options
--	Languagefile for the whole GUI Dialog
--  
--  v0.85:
--  	Added new Tab for BWL
--  	Modified some Functions from Tabbing
--  	Added Dropdown Handling
--
-- -------------------------------------------------------------------- --

LVBMGUI_DEBUG = false;
LVBMGUI_VERSION = "0.85";
LVBMGUI_TABCOUNT = 4;
LVBMGUI_LINES_TO_DISPLAY = 10;

----------------------------
-- DO NOT CHANGE ANYTHING --
----------------------------

LVBMSETUPFRAME_SUBFRAMES = { "LVBMBossModListFrame", "LVBMOptionsFrame" };
UIPanelWindows["LVBMBossModFrame"] = { area = "left",	pushable = 2,	whileDead = 1 };

local datasetInstance = {};

function LVBMBossModFrame_ShowSubFrame(frameName)
	for index, value in LVBMSETUPFRAME_SUBFRAMES do
		if ( value == frameName ) then
			getglobal(value):Show()
		else
			getglobal(value):Hide();
		end	
	end 
end

function LVBMBossModFrame_ShowDropdown(mNameId, button)
	HideDropDownMenu(1);

	LVBossModDropDownFrame.initialize = LVBMBossModFrameDropDown_Initialize;
	LVBossModDropDownFrame.displayMode = "MENU";
	LVBossModDropDownFrame.Name = datasetInstance[mNameId].Key or false;
	LVBossModDropDownFrame.button = button;

	ToggleDropDownMenu(1, nil, LVBossModDropDownFrame, "cursor");
end

function LVBMBossModFrameDropDown_Initialize()
--[[  
List of button attributes
======================================================
info.text = [STRING]  				-- The text of the button
info.value = [ANYTHING]  			-- The value that UIDROPDOWNMENU_MENU_VALUE is set to when the button is clicked
info.func = [function()]  			-- The function that is called when you click the button
info.variable = [STRING (name of a boolean var)]	-- a string which contains the name of a boolean variable, used to check the button or not

info.disabled = [nil, 1]  --  Disable the button and show an invisible button that still traps the mouseover event so menu doesn't time out
info.hasArrow = [nil, 1]  --  Show the expand arrow for multilevel menus
info.hasColorSwatch = [nil, 1]  --  Show color swatch or not, for color selection
info.r = [1 - 255]  --  Red color value of the color swatch
info.g = [1 - 255]  --  Green color value of the color swatch
info.b = [1 - 255]  --  Blue color value of the color swatch
info.textR = [1 - 255]  --  Red color value of the button text
info.textG = [1 - 255]  --  Green color value of the button text
info.textB = [1 - 255]  --  Blue color value of the button text
info.swatchFunc = [function()]  --  Function called by the color picker on color change
info.hasOpacity = [nil, 1]  --  Show the opacity slider on the colorpicker frame
info.opacity = [0.0 - 1.0]  --  Percentatge of the opacity, 1.0 is fully shown, 0 is transparent
info.opacityFunc = [function()]  --  Function called by the opacity slider when you change its value
info.cancelFunc = [function(previousValues)] -- Function called by the colorpicker when you click the cancel button (it takes the previous values as its argument)
info.notClickable = [nil, 1]  --  Disable the button and color the font white
info.notCheckable = [nil, 1]  --  Shrink the size of the buttons and don't display a check box
info.owner = [Frame]  --  Dropdown frame that "owns" the current dropdownlist
info.keepShownOnClick = [nil, 1]  --  Don't hide the dropdownlist after a button is clicked
info.tooltipTitle = [nil, STRING] -- Title of the tooltip shown on mouseover
info.tooltipText = [nil, STRING] -- Text of the tooltip shown on mouseover
info.justifyH = [nil, "CENTER"] -- Justify button text
info.arg1 = [ANYTHING] -- This is the first argument used by info.func
info.arg2 = [ANYTHING] -- This is the second argument used by info.func
info.textHeight = [NUMBER] -- font height for button text
]]	

	if( UIDROPDOWNMENU_MENU_LEVEL == 1 ) then
		local info = {};

		UIDropDownMenu_AddButton{						-- Title
			text = LVBM.AddOns[LVBossModDropDownFrame.Name].Name.." "..LVBMGUI_DROPDOWNMENU_TITLE,
			notCheckable = 1,
			isTitle = 1,
		}
--		UIDropDownMenu_AddButton{ notCheckable = 1 }				-- Space --> sucks =[

		if( LVBossModDropDownFrame.button == "RightButton" ) then		-- Only show if right clicked
			info["text"] = LVBMGUI_ENABLE_ADDON;	-- Enable/Disable AddOn
			if( LVBM.AddOns[LVBossModDropDownFrame.Name].Options.Enabled ) then info["checked"] = 1; else info["checked"] = nil; end
			info["func"] = function() 
					if( LVBM.AddOns[LVBossModDropDownFrame.Name].Options.Enabled ) then 
						SlashCmdList[LVBossModDropDownFrame.Name]("off");
					else 
						SlashCmdList[LVBossModDropDownFrame.Name]("on");
					end 
					LVBMBossModList_Update();
						end;
			info["keepShownOnClick"] = 1;
			UIDropDownMenu_AddButton(info);
	
			info["text"] = LVBMGUI_ENABLE_ANNOUNCE;	-- Enable/Disable Announce
			if( LVBM.AddOns[LVBossModDropDownFrame.Name].Options.Announce ) then info["checked"] = 1; else info["checked"] = nil; end
			info["func"] = function() 
					if( LVBM.AddOns[LVBossModDropDownFrame.Name].Options.Announce ) then 
						SlashCmdList[LVBossModDropDownFrame.Name]("announce off"); 
					else 
						SlashCmdList[LVBossModDropDownFrame.Name]("announce on");
					end 
					LVBMBossModList_Update();
				       end;
			info["keepShownOnClick"] = 1;
			UIDropDownMenu_AddButton(info);
		end
			
		local infoTable = LVBM.AddOns[LVBossModDropDownFrame.Name].DropdownMenu;
		
		if( infoTable ~= nil ) then
			
			for key, info in infoTable do
				if( info.variable ~= nil ) then 
					if( table.getglobal(info.variable) == true ) then		info.checked = 1; 
					elseif( table.getglobal(info.variable) == false ) then 	info.checked = nil; 
					end 
				end
				info.keepShownOnClick = 1;
				UIDropDownMenu_AddButton( info );
			end
		end
		
	else
		LVBMDebugMsg("[Debug][Funktionsaufruf]: LVBMBossModFrameDropDown_Initialize() - UIDROPDOWNMENU_MENU_LEVEL ~= 1");
	end

end

function LVBMBossModFrame_OnLoad()
	LVBMDebugMsg("[Debug][Funktionsaufruf]: LVBMBossModList_OnLoad()");

	PanelTemplates_SetNumTabs(this, LVBMGUI_TABCOUNT);						-- Anzahl Tabs // Naxx, AQ, etc.
	LVBMBossModFrame.selectedTab = 1;
	PanelTemplates_UpdateTabs(this);
	
	LVBMBossModFrame.selectedBossMod = 0;
end

function LVBMBossModFrame_OnShow()
	LVBMDebugMsg("[Debug][Funktionsaufruf]: LVBMBossModList_OnShow()");

	LVBMBossModFrame.showLVBMBossModList = 1;
	LVBMBossModFrame_Update();
	UpdateMicroButtons();
	PlaySound("igMainMenuOpen");
end

function LVBMBossModFrame_Update()
	LVBMDebugMsg("[Debug][Funktionsaufruf]: LVBMBossModFrame_Update()");

	LVBMBossModFrameTopLeft:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-General-TopLeft");
	LVBMBossModFrameTopRight:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-General-TopRight");
	LVBMBossModFrameBottomLeft:SetTexture("Interface\\FriendsFrame\\UI-FriendsFrame-BotLeft");
	LVBMBossModFrameBottomRight:SetTexture("Interface\\FriendsFrame\\UI-FriendsFrame-BotRight");

	if ( LVBMBossModFrame.selectedTab >= 1 and LVBMBossModFrame.selectedTab <= 3 ) then
		LVBMBossModFrameTitleText:SetText(getglobal("LVBMGUI_TAB_"..LVBMBossModFrame.selectedTab.."_TITLE_TEXT") );
		LVBMBossModList_Update();
		LVBMBossModFrame_ShowSubFrame("LVBMBossModListFrame");

	elseif( LVBMBossModFrame.selectedTab == 4 ) then						-- Options Frame -- need to load another frame
		LVBMBossModFrameBottomLeft:SetTexture("Interface\\FriendsFrame\\GuildFrame-BotLeft");
		LVBMBossModFrameBottomRight:SetTexture("Interface\\FriendsFrame\\GuildFrame-BotRight")

		LVBMBossModFrameTitleText:SetText(getglobal("LVBMGUI_TAB_"..LVBMBossModFrame.selectedTab.."_TITLE_TEXT"));
		LVBMBossModFrame_ShowSubFrame("LVBMOptionsFrame");

	end
end

function LVBMBossModFrame_OnHide()
	UpdateMicroButtons();
	PlaySound("igMainMenuClose");
	for index, value in LVBMSETUPFRAME_SUBFRAMES do
		getglobal(value):Hide();
	end
end

function LVBMBossModList_Update()
	LVBMDebugMsg("[Debug][Funktionsaufruf]: LVBMBossModList_Update() - Tab="..LVBMBossModFrame.selectedTab);

	local numLVBMBossMod = 0;
	datasetInstance = {};

	for key, value in LVBM.SortedAddOns do
		if( LVBM.AddOns[value].Instance == nil ) then
			-- Buggy Template
			LVBMDebugMsg("[Debug][TemplateFehler]: key="..value );
			
		elseif( LVBM.AddOns[value].Instance == "Naxxramas" and LVBMBossModFrame.selectedTab == 1 ) then			-- Naxxramas
			numLVBMBossMod = numLVBMBossMod + 1;
			datasetInstance[numLVBMBossMod] = {};
			datasetInstance[numLVBMBossMod]["Key"] = value; 
			datasetInstance[numLVBMBossMod]["Name"] = LVBM.AddOns[value].Name;
			datasetInstance[numLVBMBossMod]["Description"] = LVBM.AddOns[value].Description;

		elseif( LVBM.AddOns[value].Instance == "Ahn'Qiraj" and LVBMBossModFrame.selectedTab == 2 ) then			-- Ahn'Qiraj
			numLVBMBossMod = numLVBMBossMod + 1;
			datasetInstance[numLVBMBossMod] = {};
			datasetInstance[numLVBMBossMod]["Key"] = value; 
			datasetInstance[numLVBMBossMod]["Name"] = LVBM.AddOns[value].Name;
			datasetInstance[numLVBMBossMod]["Description"] = LVBM.AddOns[value].Description;

		elseif( LVBM.AddOns[value].Instance == "Blackwing Lair" and LVBMBossModFrame.selectedTab == 3 ) then		-- BWL
			numLVBMBossMod = numLVBMBossMod + 1;
			datasetInstance[numLVBMBossMod] = {};
			datasetInstance[numLVBMBossMod]["Key"] = value; 
			datasetInstance[numLVBMBossMod]["Name"] = LVBM.AddOns[value].Name;
			datasetInstance[numLVBMBossMod]["Description"] = LVBM.AddOns[value].Description;

		end
	end
	
	local nameLocationText;
	local infoText;
	local bossButton;
	
	local bossOffset = FauxScrollFrame_GetOffset(LVBMBossModFrameLVBMBossModScrollFrame);
	local bossIndex;
	for i=1, LVBMGUI_LINES_TO_DISPLAY, 1 do
		bossIndex = bossOffset + i;

		nameLocationText = getglobal("LVBMBossModButton"..i.."ButtonTextNameLocation");
		infoText = getglobal("LVBMBossModButton"..i.."ButtonTextInfo");

		if( datasetInstance[bossIndex] ~= nil ) then
			if( LVBM.AddOns[datasetInstance[bossIndex].Key].Options.Enabled ) then
				nameLocationText:SetText(datasetInstance[bossIndex]["Name"]);					-- Main Text
			else
				nameLocationText:SetText("|cff999999"..datasetInstance[bossIndex]["Name"].."|r");
			end 

			infoText:SetText( LVBMCutTextafter(datasetInstance[bossIndex]["Description"], 54) );			-- Sub Text
			
			setglobal("LVBMBossModButton"..i.."TTText", datasetInstance[bossIndex]["Name"]);
			setglobal("LVBMBossModButton"..i.."TTDesc", datasetInstance[bossIndex]["Description"]);
		end

		bossButton = getglobal("LVBMBossModButton"..i);
		bossButton:SetID(bossIndex);
		
		-- Update the highlight
		if ( bossIndex == LVBMBossModFrame.selectedBossMod ) then
			bossButton:LockHighlight();
		else
			bossButton:UnlockHighlight();
		end
		
		if ( bossIndex > numLVBMBossMod ) then
			bossButton:Hide();
		else
			bossButton:Show();
		end

	end

	-- Update the Buttons on the Bottom Frame for Options
	if( LVBMBossModFrame.selectedBossMod > 0 ) then
		local shortkey = datasetInstance[LVBMBossModFrame.selectedBossMod].Key;
		local shortvar = LVBM.AddOns[shortkey];
	
		if( LVBM.AddOns[shortkey].Options.Enabled ) then 					-- Button 1 -> on/off
			LVBMBossModListFrameButton1:SetText( LVBMGUI_DISABLE_ADDON );
			LVBMBossModListFrameButton1:SetScript("OnClick", function() SlashCmdList[shortkey]("off"); LVBMBossModList_Update(); end);
		else
			LVBMBossModListFrameButton1:SetText( LVBMGUI_ENABLE_ADDON );
			LVBMBossModListFrameButton1:SetScript("OnClick", function() SlashCmdList[shortkey]("on"); LVBMBossModList_Update(); end );
		end
		LVBMBossModListFrameButton1:Enable();

		LVBMBossModListFrameButton2:SetText( LVBMGUI_STOP_ADDON );				-- Button 2 -> stop
		LVBMBossModListFrameButton2:SetScript("OnClick", function() SlashCmdList[shortkey]("stop"); LVBMBossModList_Update(); end );
		LVBMBossModListFrameButton2:Enable();

		if( LVBM.AddOns[shortkey].Options.Announce ) then					-- Button 3 -> Announce on/off
			LVBMBossModListFrameButton3:SetText( LVBMGUI_DISABLE_ANNOUNCE );
			LVBMBossModListFrameButton3:SetScript("OnClick", function() SlashCmdList[shortkey]("announce off"); LVBMBossModList_Update(); end);
		else
			LVBMBossModListFrameButton3:SetText( LVBMGUI_ENABLE_ANNOUNCE );
			LVBMBossModListFrameButton3:SetScript("OnClick", function() SlashCmdList[shortkey]("announce on"); LVBMBossModList_Update(); end );
		end
		LVBMBossModListFrameButton3:Enable();

		if( LVBM.AddOns[shortkey].DropdownMenu ~= nil ) then
			LVBMBossModListFrameButton4:SetText( LVBMGUI_SHOW_DROPDOWNMENU );				-- Button 4 -> Show Dropdown
			LVBMBossModListFrameButton4:SetScript("OnClick", function() LVBMBossModFrame_ShowDropdown( LVBMBossModFrame.selectedBossMod ); end );
			LVBMBossModListFrameButton4:Enable();
		else 
			LVBMBossModListFrameButton4:SetText( LVBMGUI_SHOW_DROPDOWNMENU );
			LVBMBossModListFrameButton4:Disable();
		end
	else
		-- Set Text and Grey out Buttons
		LVBMBossModListFrameButton1:SetText( LVBMGUI_ENABLE_ADDON );
		LVBMBossModListFrameButton1:Disable();

		LVBMBossModListFrameButton2:SetText( LVBMGUI_STOP_ADDON );
		LVBMBossModListFrameButton2:Disable();

		LVBMBossModListFrameButton3:SetText( LVBMGUI_ENABLE_ANNOUNCE );
		LVBMBossModListFrameButton3:Disable();

		LVBMBossModListFrameButton4:SetText( LVBMGUI_SHOW_DROPDOWNMENU );
		LVBMBossModListFrameButton4:Disable();
	end
	
	-- ScrollFrame stuff
	FauxScrollFrame_Update(LVBMBossModFrameLVBMBossModScrollFrame, numLVBMBossMod, LVBMGUI_LINES_TO_DISPLAY, 34 );
end

function LVBMBossModFrame_OnEvent()
	-- Currently Nothing ..
end

function LVBMBossModButton_OnClick(button)			-- Called on Mouseclick on a Listitem
	LVBMDebugMsg("[Debug][Funktionsaufruf]: LVBMBossModButton_OnClick(button) - Button="..button.." ID="..this:GetID());

	if ( button == "LeftButton" ) then
		LVBMBossModFrame.selectedBossMod = this:GetID();
		LVBMBossModList_Update();
	else
		LVBMBossModFrame.selectedBossMod = this:GetID();
		LVBMBossModList_Update();
		LVBMBossModFrame_ShowDropdown( this:GetID(), button);
	end
end


-- Debug Output for Developement Help
function LVBMDebugMsg(msg)
	if ( LVBMGUI_DEBUG ) then
		DEFAULT_CHAT_FRAME:AddMessage(msg);
	end
end


-- Function to Change Minimap Position using the LVBM.Options.MinimapButton Variable
function LVBMMinimapButton_Move()
	LVBMMinimapButton:SetPoint("CENTER", "Minimap", "CENTER", (LVBM.Options.MinimapButton.Radius * cos(LVBM.Options.MinimapButton.Position)), 
							(LVBM.Options.MinimapButton.Radius * sin(LVBM.Options.MinimapButton.Position)));
end


-- Function to Update changes at the Statusbars
function LVBMGuiUpdateStatusbars()
	if( getglobal("LVBM_StatusBarTimer1") == nil ) then return false; end		-- only a problem if statusbars.xml no loaded
	for i = 1, 10 do
		getglobal("LVBM_StatusBarTimer"..i):SetStatusBarColor(	LVBM.Options.StatusBarColor.r, 
									LVBM.Options.StatusBarColor.g, 
									LVBM.Options.StatusBarColor.b, 
									LVBM.Options.StatusBarColor.a	);
	end
	LVBM_StatusBarTimerDrag:SetStatusBarColor(	LVBM.Options.StatusBarColor.r, 
							LVBM.Options.StatusBarColor.g, 
							LVBM.Options.StatusBarColor.b, 
							LVBM.Options.StatusBarColor.a	);
end


-- Function to reset MiniMap Button
function LVBMMinimapButton_SetDefaultValues()
	LVBM.Options["MinimapButton"] = {};
	LVBM.Options.MinimapButton["Position"]	= 225;
	LVBM.Options.MinimapButton["Radius"]	= 78.1;
	LVBM.Options.MinimapButton["Enabled"]	= true;
	LVBMMinimapButton_Move();
end


-- Function to Cut Text blocks after len and only on spaces
function LVBMCutTextafter(tstring, len)
	if( string.len(tstring) < len ) then return tstring; end
	local last = 0;
	local i = 0
	while(true)do
		i = string.find(tstring, " ", i+1) 
		if (i == nil) then 
			break;

		elseif( i < len) then
			last = i - 1;
			
		elseif( i > len) then
			break;
		end
	end

	return string.sub(tstring, 1, last).."...";
end

-- Damm Lua ... GetGlobal from Table
function table.getglobal(var)
	return(loadstring("return "..tostring(var))());
end


-- Damm Lua ... Math Round must be standard .. but not in lua ... 
function math.round( num,  idp )
	-- init numbers
	local idp = idp or 1
	idp = idp + 1
	--get rest
	if tonumber(num) then
		local dpoint = 0
		local count = 0
		local strnum = ""
		for i in string.gfind( tostring( num ), "(.)") do
			if dpoint ~= 0 then
				count = count + 1
				if count == idp+1 then
					break
				end
			end
			if i == "." then
				dpoint = 1
			end
			strnum = strnum..i
		end
		if dpoint == 0 then
			return num
		end
		-- get number to round
		local toround = tonumber( string.sub( strnum, -1, -1 ) )
		-- evaluate ( if x < 5 then x = 0 ) else x = 10
		if toround < 5 then
			-- remove last number, done
			return  tonumber( string.sub( strnum, 1, -2 ) )
		end
		-- else remove last and add one to before
		local ibefore = string.sub( strnum, -2, -2 )
		-- if dot then only add one and return
		if ibefore == "." then
			return tonumber( string.sub( strnum, 1, -3 ) ) + 1
		end
		ibefore = tonumber( ibefore )
		ibefore = ibefore + 1
		if ibefore == 10 then
			return math.round( tonumber( string.sub( strnum, 1, -2 ) ), idp-1 )
		end
		return tonumber( string.sub( strnum, 1, -3 )..ibefore )
	end
end


